#!/bin/bash
VER=2.0

#--[ Intro ]--------------------------------------------------------#
#                                                                   #
# Tur-Archiver 2.x is a total rewrite from 1.x. It can copy, move   #
# or make symlinks from whatever you want, to wherever you want.    #
#                                                                   #
# It is NOT a spacemaker and shouldnt be used as such.              #
#                                                                   #
#--[ Installation ]-------------------------------------------------#
#                                                                   #
# Copy tur-archiver.sh to /glftpd/bin and chmod it to 700 or so.    #
#                                                                   #
# The included file_date binary is only needed if you wish to set a #
# time, in minutes, before this script touches it. If so, either    #
# copy the binary to /glftpd/bin or compile the source:             #
# gcc -o /glftpd/bin/file_date file_date.c                          #
#                                                                   #
#--[ Settings ]-----------------------------------------------------#
#                                                                   #
# ROOT       = Simple the root of your glftpd's site dir.           #
#              This is automatically added to the front of each     #
#              source and destination dir you soon choose.          #
#                                                                   #
#              You can set this to "" if you'd rather use full      #
#              paths in MOVE= below except if you want to create    #
#              symlinks (see SYM_ROOT at the bottom).               #
#                                                                   #
# today      = Incase you wish to use this in dated dirs, this will #
#              automatically set $today to todays dated dir (MMDD). #
#              The we can use $today in MOVE below.                 #
#                                                                   #
# MOVE       = Ah, the heart of it all.                             #
#              Format is:                                           #
#              Where_To_Look:What_To_Look_For:Where_To_Put_It       #
#                                                                   #
#              Where_To_Look is the source section, relative to     #
#              $ROOT, which it will look for stuff in.              #
#              If you must use a space here, use [space] instead.   #
#                                                                   #
#              What_To_Look_For is a standard egrep query. That     #
#              means you should know something about basic egrep to #
#              use it at best. For instance GROUPS means it will    #
#              match the word GROUPS anywhere in the release.       #
#                                                                   #
#              ^GROUPS means the dir must START with GROUPS.        #
#              GROUPS$ means the dir must END with GROUPS.          #
#              ^GROUPS$ means the dir must be named exactly GROUPS. #
#                                                                   #
#              Any odd chars must be escaped. For instance a .      #
#              would mean "anything" whereas \. means a real dot.   #
#                                                                   #
#              Do NOT use spaces in this one. If you must, use "\ " #
#              (ie, escape the space).                              #
#                                                                   #
#              If you want to search for - . OR _  do [\_-\.]       #
#              For instance [\_-\.]internal[\_-\.]                  #
#                                                                   #
#              Seperate different querys with a | sign.             #
#                                                                   #
#              Where_To_Put_It is where to put the resulting dir.   #
#              If you must use a space here, use [space] instead.   #
#                                                                   #
# CASE_SENSITIVE= TRUE/FALSE. Do you want the What_To_Look_For to   #
#                 be case sensitive or not? (egrep -i)              #
#                                                                   #
# WHAT_TO_DO = What would you like to do in the destination dir?    #
#              S = Make a symlink.                                  #
#              C = Copy the release here.                           #
#              M = Move the release here.                           #
#                                                                   #
# EXCLUDE    = This too is a default egrep line, case sensitive.    #
#              Anything matched here will be totally ignored        #
#              even if it matches a hit.                            #
#                                                                   #
# MINUTES_OLD= How old, in minutes, must the release be before we   #
#              do anything to it?   Set to "" to move instantly.    #
#              (setting it to 0 will also, but script will run      #
#              faster if you set it to "" since it wont even check) #
#                                                                   #
# CHECK_FOR  = Anything that must be in the dir before we move it.  #
#                                                                   #
#              By default, its "\ COMPLETE\ \)\ \-\ \[xXx\]"        #
#              which translates to " COMPLETE ) - [xXx]"            #
#              That happens to be the end of the default COMPLETE   #
#              line that zipscript-c outputs. Just change sitename. #
#                                                                   #
#              If the release has multiple CD's, each CDx dir must  #
#              contain this (We'll only check CD1 -> CD9)           #
#                                                                   #
#              This is also an egrep search, case sensitive, so if  #
#              you want to add other stuff that should also make    #
#              the release ok to move just add it here, | seperated #
#                                                                   #
#              Set to "" to disable or put a # infront of it.       #
#                                                                   #
# HOW_TO_SYMLINK = This is how it will run if WHAT_TO_DO is S.      #
# HOW_TO_MOVE    = This is how it will run if WHAT_TO_DO is M.      #
# HOW_TO_COPY    = This is how it will run if WHAT_TO_DO is C.      #
#                                                                   #
# SYMLINK_CLEAR  = TRUE/FALSE. This is only valid if WHAT_TO_DO is  #
#                  set to S.                                        #
#                  If this is TRUE, it will delete every            #
#                  Where_To_Put_It dir before it starts and then    #
#                  recreate them with 755 permissions.              #
#                  Its to make sure there are no "dead" symlinks.   #
#                  Be careful with this, as it will rm -f any dir   #
#                  you set as destination for the symlinks.         #
#                                                                   #
#                  If you dont want 755 perms, search for -m755     #
#                  below and change it to whatever you want.        #
#                                                                   #
# SYM_ROOT   = When copying or moving, we can use the full path     #
#              but when making symlinks, we cant point it towards   #
#              /glftpd/site/blabla since it wont be found from      #
#              inside glftpd's chroot.                              #
#              Therefor we must set a specific root for that here.  #
#              Shouldnt need to touch it really.                    #
#                                                                   #
# FILE_DATE  = Incase MINUTES_OLD is set, we use the included       #
#              file_date binary to check its age. Here you specify  #
#              it location.                                         #
#                                                                   #
# DATE_BIN   = Incase MINUTES_OLD is set, we need a GNU compliant   #
#              date binary that supports the -d option.             #
#              Most users can leave it at just "date".              #
#              FBSD users will want to download sh-utils. It will   #
#              install a binary called gdate which you must specify #
#              here.                                                #
#                                                                   #
# DEBUG      = TRUE/FALSE. Just a precation. You must set this to   #
#              FALSE for it to actually do anything. With it on     #
#              TRUE, its the same as if you used the 'debug' arg.   #
#                                                                   #
#--[ Running it ]---------------------------------------------------#
#                                                                   #
# Running it with the argument 'debug' (without the '') will not do #
# anything except show you what it WOULD have done. I recomend you  #
# use this everytime you make a change in any part.                 #
#                                                                   #
# Running it without args will make a live run. Shouldnt output     #
# anything then.                                                    #
#                                                                   #
#--[ Changelog ]----------------------------------------------------#
#                                                                   #
# 2.0   : Total rewrite of the 1.x series. Can do a lot more now.   #
#                                                                   #
#--[ Contact ]------------------------------------------------------#
#                                                                   #
# http://www.grandis.nu/glftpd <-> http://grandis.mine.nu/glftpd    #
#                                                                   #
#--[ Configuration ]------------------------------------------------#

ROOT=/glftpd/site

today="$(date +%m%d)"

MOVE="
/DIVX:^The\.|\-BP$:/Archive/DIVX/SomeDir
/SVCD:Flix$|[\_-\.]Int[\_-\.]:/Archive/SVCD/SomeDir
/0DAYS/$today:^3D\.|\-SomeCrapGroup$:/Archive/SVCD/SomeDir
"

CASE_SENSITIVE=FALSE

WHAT_TO_DO=S

EXCLUDE="^lost\+found$|^GROUPS$|^\_PRE$"

MINUTES_OLD=200

CHECK_FOR="\ COMPLETE\ \)\ \-\ \[xXx\]"

## Advanced options.

HOW_TO_SYMLINK="ln -f -s"
HOW_TO_MOVE="mv -f"
HOW_TO_COPY="cp -f"

SYMLINK_CLEAR=TRUE
SYM_ROOT="/site"

FILE_DATE=/glftpd/bin/file_date
DATE_BIN=date

DEBUG=TRUE


#--[ Script Start ]-------------------------------------------------#

## Set DEBUG=TRUE if first argument is debug or DEBUG
if [ "$1" = "debug" -o "$1" = "DEBUG" ]; then
  DEBUG="TRUE"
fi

## Set how we want to egrep.
if [ "$CASE_SENSITIVE" = "TRUE" ]; then
  EGREP="egrep"
else
  EGREP="egrep -i"
fi

## Move, copy or make symlinks. Heres where its done.
proc_copy() {
  if [ "$DEBUG" = "TRUE" ]; then
    echo "$HOW_TO_COPY \"$FROM_DIR/$releasename\" \"$TO_DIR\""
  else
    $HOW_TO_COPY "$FROM_DIR/$releasename" "$TO_DIR"
  fi
}
proc_move() {
  if [ "$DEBUG" = "TRUE" ]; then
    echo "$HOW_TO_MOVE \"$FROM_DIR/$releasename\" \"$TO_DIR\""
  else
    $HOW_TO_MOVE "$FROM_DIR/$releasename" "$TO_DIR"
  fi
}
proc_symlink() {
  if [ "$DEBUG" = "TRUE" ]; then
    echo "$HOW_TO_SYMLINK \"$FROM_DIRSYM/$releasename\" \"$TO_DIR/$releasename\""
  else
    $HOW_TO_SYMLINK "$FROM_DIRSYM/$releasename" "$TO_DIR/$releasename"
  fi
}

## Procedure for checking if the release is old enough.
proc_checkold() {
  if [ "$MINUTES_OLD" ] && [ "$SKIP" != "YES" ]; then
    REL_DATE="`$DATE_BIN -d "$($FILE_DATE $releasename)" +%s`"
    if [ -z "$REL_DATE" ]; then
      SKIP=YES
      if [ "$DEBUG" = "TRUE" ]; then
        echo "Skipping move of $releasename - Seems to be from right now or in the future..."
      fi
    else
      REL_SEC_OLD="`echo "$NOW_DATE - $REL_DATE" | bc -l | cut -d '.' -f1`"
      REL_MIN_OLD="`echo "$REL_SEC_OLD / 60" | bc -l | cut -d '.' -f1`"
      if [ "$REL_MIN_OLD" -lt "$MINUTES_OLD" ]; then
        SKIP=YES
        if [ "$DEBUG" = "TRUE" ]; then
          echo "Skipping move of $releasename - Only $REL_MIN_OLD minutes old."
        fi
      fi
    fi
  fi
}

## Procedure for checking for other stuff in the release to OK it.
proc_checkfor() {
  if [ "$CHECK_FOR" ] && [ "$SKIP" != "YES" ]; then
    if [ "`ls -1 $releasename | grep "^[cC][dD][1-9]$"`" ]; then
      for each_cd in `ls -1 $releasename | grep "^[cC][dD][1-9]$"`; do
        if [ -z "`ls -1 $releasename/$each_cd | egrep "$CHECK_FOR"`" ]; then
          if [ "$DEBUG" = "TRUE" ]; then
            echo "Skipping $releasename - $each_cd does not seem complete."
          fi
          SKIP=YES
          break
        fi
      done
    else
      if [ -z "`ls -1 $releasename | egrep "$CHECK_FOR"`" ]; then
        if [ "$DEBUG" = "TRUE" ]; then
          echo "Skipping $releasename - Does not seem complete."
        fi
        SKIP=YES
      fi
    fi
  fi
}

if [ "$MINUTES_OLD" ]; then
  NOW_DATE="`$DATE_BIN +%s`"
  if [ ! -x "$FILE_DATE" ]; then
    echo "Error. MINUTES_OLD is defined but cant find/execute file_date from $FILE_DATE"
    exit 1
  fi
fi

## If we use symlinks and SYMLINK_CLEAR is TRUE, delete all TO_DIRs and recreate
if [ "$WHAT_TO_DO" = "S" ] && [ "$SYMLINK_CLEAR" = "TRUE" ]; then
  for rawdata in $MOVE; do
    TO_DIR="$ROOT`echo "$rawdata" | cut -d ':' -f3 | sed -e 's/\[space\]/ /g'`"
    if [ "$TO_DIR" != "$LAST_TO_DIR" ]; then
      if [ -d "$TO_DIR" ]; then
        if [ "$DEBUG" = "TRUE" ]; then
          echo "Deleting and recreating $TO_DIR"
        else
          rm -rf "$TO_DIR"
          mkdir -m755 "$TO_DIR"
        fi
      fi
    fi
    LAST_TO_DIR="$TO_DIR"
  done
fi  

## Go. Main loop.
for rawdata in $MOVE; do
  unset SKIP
echo "$rawdata"
  FROM_DIR="$ROOT`echo "$rawdata" | cut -d ':' -f1 | sed -e 's/\[space\]/ /g'`"
  GRAB_WHAT="`echo "$rawdata" | cut -d ':' -f2`"
  TO_DIR="$ROOT`echo "$rawdata" | cut -d ':' -f3 | sed -e 's/\[space\]/ /g'`"

echo "$FROM_DIR"
echo "$TO_DIR"

  ## Set a specific path incase we want symlinks.
  if [ "$WHAT_TO_DO" = "S" ]; then
    FROM_DIRSYM="$SYM_ROOT`echo "$rawdata" | cut -d ':' -f1 | sed -e 's/\[space\]/ /g'`"
  fi

  if [ ! -d "$FROM_DIR" ]; then
    echo "Error. \"$FROM_DIR\" does not exist."
    SKIP=YES
  fi
  if [ ! -d "$TO_DIR" ]; then
    echo "Error. \"$TO_DIR\" does not exist."
    SKIP=YES
  fi

  if [ "$SKIP" != "YES" ]; then
    if [ "$DEBUG" = "TRUE" ]; then
      echo ""
      echo "Entering $FROM_DIR - Looking for \"$GRAB_WHAT\""
    fi

    cd "$FROM_DIR"

    for releasename in `ls -1 | $EGREP "$GRAB_WHAT" | egrep -v "$EXCLUDE" | tr ' ' '^'`; do
      unset SKIP
      releasename="`echo "$releasename" | tr '^' ' '`"

      proc_checkold
      proc_checkfor

      if [ "$SKIP" != "YES" ]; then
        case $WHAT_TO_DO in
          [mM]) proc_move;;
          [cC]) proc_copy;;
          [sS]) proc_symlink;;
        esac
      fi

    done
  fi

done

if [ "$DEBUG" = "TRUE" ]; then
  echo ""
  echo "All done."
fi

exit 0